@@ -27,11 +27,12 @@
 #include <kjob.h>
 
 #include <QApplication>
+#include <QDBusConnection>
 #include <QIcon>
 
 Q_GLOBAL_STATIC(KSharedUiServerProxy, serverProxy)
 
-class KUiServerJobTracker::Private
+class Q_DECL_HIDDEN KUiServerJobTracker::Private
 {
 public:
     Private(KUiServerJobTracker *parent)
@@ -43,7 +44,11 @@ public:
 
     void _k_killJob();
 
+    static void updateDestUrl(KJob *job, org::kde::JobViewV2 *jobView);
+
     QHash<KJob *, org::kde::JobViewV2 *> progressJobView;
+
+    QMetaObject::Connection serverRegisteredConnection;
 };
 
 void KUiServerJobTracker::Private::_k_killJob()
@@ -59,6 +64,14 @@ void KUiServerJobTracker::Private::_k_killJob()
     }
 }
 
+void KUiServerJobTracker::Private::updateDestUrl(KJob *job, org::kde::JobViewV2 *jobView)
+{
+    const QVariant destUrl = job->property("destUrl");
+    if (destUrl.isValid()) {
+        jobView->setDestUrl(QDBusVariant(destUrl));
+    }
+}
+
 KUiServerJobTracker::KUiServerJobTracker(QObject *parent)
     : KJobTrackerInterface(parent), d(new Private(this))
 {
@@ -82,6 +95,21 @@ void KUiServerJobTracker::registerJob(KJob *job)
         return;
     }
 
+    // Watch the server registering/unregistering and re-register the jobs as needed
+    if (!d->serverRegisteredConnection) {
+        d->serverRegisteredConnection = connect(serverProxy(), &KSharedUiServerProxy::serverRegistered, this, [this]() {
+            // Remember the list of jobs to re-register and then delete the old ones
+            const QList<KJob *> staleJobs = d->progressJobView.keys();
+
+            qDeleteAll(d->progressJobView);
+            d->progressJobView.clear();
+
+            for (KJob *job : staleJobs) {
+                registerJob(job);
+            }
+        });
+    }
+
     const QString appName = QCoreApplication::applicationName();
     // This will only work if main() used QIcon::fromTheme.
     QString programIconName = QApplication::windowIcon().name();
@@ -107,17 +135,14 @@ void KUiServerJobTracker::registerJob(KJob *job)
             return;
         }
 
-        QObject::connect(jobView, SIGNAL(cancelRequested()), this,
-                         SLOT(_k_killJob()));
-        QObject::connect(jobView, SIGNAL(suspendRequested()), job,
-                         SLOT(suspend()));
-        QObject::connect(jobView, SIGNAL(resumeRequested()), job,
-                         SLOT(resume()));
+        QObject::connect(jobView, SIGNAL(cancelRequested()),
+                         this, SLOT(_k_killJob()));
+        QObject::connect(jobView, &org::kde::JobViewV2::suspendRequested,
+                         job, &KJob::suspend);
+        QObject::connect(jobView, &org::kde::JobViewV2::resumeRequested,
+                         job, &KJob::resume);
 
-        QVariant destUrl = job->property("destUrl");
-        if (destUrl.isValid()) {
-            jobView->setDestUrl(QDBusVariant(destUrl));
-        }
+        d->updateDestUrl(job, jobView);
 
         if (!jobWatch) {
             //qCDebug(KJOBWIDGETS) << "deleted out from under us when creating the dbus interface";
@@ -146,6 +171,8 @@ void KUiServerJobTracker::unregisterJob(KJob *job)
 
     org::kde::JobViewV2 *jobView = d->progressJobView.take(job);
 
+    d->updateDestUrl(job, jobView);
+
     jobView->setError(job->error());
 
     if (job->error()) {
@@ -165,6 +192,8 @@ void KUiServerJobTracker::finished(KJob *job)
 
     org::kde::JobViewV2 *jobView = d->progressJobView.take(job);
 
+    d->updateDestUrl(job, jobView);
+
     jobView->setError(job->error());
 
     if (job->error()) {
@@ -252,8 +281,6 @@ void KUiServerJobTracker::totalAmount(KJob *job, KJob::Unit unit, qulonglong amo
     case KJob::Directories:
         jobView->setTotalAmount(amount, QStringLiteral("dirs"));
         break;
-    default:
-        break;
     }
 }
 
@@ -275,8 +302,6 @@ void KUiServerJobTracker::processedAmount(KJob *job, KJob::Unit unit, qulonglong
     case KJob::Directories:
         jobView->setProcessedAmount(amount, QStringLiteral("dirs"));
         break;
-    default:
-        break;
     }
 }
 
@@ -304,24 +329,32 @@ void KUiServerJobTracker::speed(KJob *job, unsigned long value)
 
 KSharedUiServerProxy::KSharedUiServerProxy()
     : m_uiserver(QStringLiteral("org.kde.JobViewServer"), QStringLiteral("/JobViewServer"), QDBusConnection::sessionBus())
+    , m_watcher(QStringLiteral("org.kde.JobViewServer"), QDBusConnection::sessionBus(), QDBusServiceWatcher::WatchForOwnerChange)
 {
     QDBusConnectionInterface *bus = QDBusConnection::sessionBus().interface();
     if (!bus->isServiceRegistered(QStringLiteral("org.kde.JobViewServer"))) {
         QDBusReply<void> reply = bus->startService(QStringLiteral("org.kde.kuiserver"));
         if (!reply.isValid()) {
             qCritical() << "Couldn't start kuiserver from org.kde.kuiserver.service:" << reply.error();
-        } else if (!bus->isServiceRegistered(QStringLiteral("org.kde.JobViewServer"))) {
-            qCDebug(KJOBWIDGETS) << "The dbus name org.kde.JobViewServer is STILL NOT REGISTERED, even after starting kuiserver. Should not happen.";
-        } else {
-            qCDebug(KJOBWIDGETS) << "kuiserver registered";
+            return;
         }
+
+        if (!bus->isServiceRegistered(QStringLiteral("org.kde.JobViewServer"))) {
+            qCDebug(KJOBWIDGETS) << "The dbus name org.kde.JobViewServer is STILL NOT REGISTERED, even after starting kuiserver. Should not happen.";
+            return;
+        }
+
+        qCDebug(KJOBWIDGETS) << "kuiserver registered";
     } else {
         qCDebug(KJOBWIDGETS) << "kuiserver found";
     }
+
+    connect(&m_watcher, &QDBusServiceWatcher::serviceOwnerChanged, this, &KSharedUiServerProxy::uiserverOwnerChanged);
 }
 
 KSharedUiServerProxy::~KSharedUiServerProxy()
 {
+
 }
 
 org::kde::JobViewServer &KSharedUiServerProxy::uiserver()
@@ -329,5 +362,17 @@ org::kde::JobViewServer &KSharedUiServerProxy::uiserver()
     return m_uiserver;
 }
 
+void KSharedUiServerProxy::uiserverOwnerChanged(const QString &serviceName, const QString &oldOwner, const QString &newOwner)
+{
+    Q_UNUSED(serviceName);
+    Q_UNUSED(oldOwner);
+
+    if (!newOwner.isEmpty()) { // registered
+        emit serverRegistered();
+    } else if (newOwner.isEmpty()) { // unregistered
+        emit serverUnregistered();
+    }
+}
+
 #include "moc_kuiserverjobtracker.cpp"
 #include "moc_kuiserverjobtracker_p.cpp"
